//******************************************************************
//******************************************************************
//** Author : Syed Salman Alam
//** Custome code for Reading, Writing and erasing data to and from SD card
//** every function is tested. It works for both SD and SDHC cards
//******************************************************************
//******************************************************************


#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"

typedef enum
{
	SD_CARD_NONE_ERROR = 0,
	SD_CARD_TYPE_SC,
	SD_CARD_TYPE_HC,
} SD_CARD_TYPE;

// Total capacity of the SD card in bytes (after initialization)
extern uint32_t totalBlocks;

//******************************************************************
//Function: to initialize (SPI) SD card pins
//Arguments: none
//******************************************************************
void SPI_SD_PIN_init(void);

//******************************************************************
//Function: to initialize SPI and  SD/SDHC  Cards
//Arguments: none
//Note: SPI_SD_PIN_init() should have been called already
//return: 1 if SD Card, 2 if SDHC card, 0 if error
//******************************************************************
SD_CARD_TYPE SD_init(void);

//******************************************************************
//Function: De-initialize SD card (when present)
//Arguments: none
//return: SD_CARD_NONE_ERROR (always)
//******************************************************************
SD_CARD_TYPE SD_close(void);

//******************************************************************
//Function: to read a single block from SD card
//param: block address
//param: p_buffer: buffer to fill with read data (should be of size 512)
//return: unsigned char; will be 0 if no error,
// otherwise the response byte will be sent
//******************************************************************
uint8_t SD_readSingleBlock(uint32_t startBlock, uint8_t *p_buffer);


//******************************************************************
//Function: to write to a single block of SD card
//param: block address
//param: p_buffer: buffer of data to write (should be of size 512)
//return: unsigned char; will be 0 if no error,
// otherwise the response byte will be sent
//******************************************************************
uint8_t SD_writeSingleBlock(uint32_t startBlock, uint8_t *p_buffer);


//*****************************************************************
//Function: to erase specified no. of blocks of SD card
//param: starting block address
//param: number of blocks to erase
//return: unsigned char; will be 0 if no error,
// otherwise the response byte will be sent
// problem it is wring FF when erasing sector
//*****************************************************************
uint8_t SD_erase (uint32_t startBlock, uint32_t totalBlocks);
void SD_erase_Modified(uint32_t startBlock, uint32_t totalBlocks);		// recomended function to use

//******************************************************************
//Function: read the capacity of the SD card in bytes
//return: card's capacity or 1GB if the operation fails
//******************************************************************
long SD_readNbrBlocks(void);

//***************************************************************************
//Function: to read multiple blocks from SD card & send every block to UART
//param: starting block address
//param: ending block address
//return: returns data in buffer[] 
// otherwise the response byte will be sent
//****************************************************************************
void Read_Sectors(uint32_t startBlock, uint32_t totalBlocks);
